/*!
 * @file main.c
 * @brief Microwave 6 Click Example.
 *
 * # Description
 * This example demonstrates the use of the Microwave 6 Click board™ 
 * by reading and displaying the results of AD conversion and motion detection.
 *
 * The demo application is composed of two sections :
 *
 * ## Application Init
 * The initialization of I2C or ADC module and log UART.
 * After driver initialization, the app calculates the reference ADC value.
 *
 * ## Application Task
 * The demo application reads the ADC results, takes an ADC sample, 
 * compares the difference between the taken samples and the ADC reference value, 
 * and reports the movement if the difference is higher/lower than the selected threshold value.
 * Results are being sent to the UART Terminal, where you can track their changes.
 *
 * @author Nenad Filipovic
 *
 */

#include "board.h"
#include "log.h"
#include "microwave6.h"

#define MICROWAVE6_THRESHOLD    500u

static microwave6_t microwave6;   /**< Microwave 6 Click driver object. */
static log_t logger;    /**< Logger object. */
static uint16_t reference = 0, old_detector = 0, raw_adc = 0;

void application_init ( void )
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    microwave6_cfg_t microwave6_cfg;  /**< Click config object. */

    /** 
     * Logger initialization.
     * Default baud rate: 115200
     * Default log level: LOG_LEVEL_DEBUG
     * @note If USB_UART_RX and USB_UART_TX 
     * are defined as HAL_PIN_NC, you will 
     * need to define them manually for log to work. 
     * See @b LOG_MAP_USB_UART macro definition for detailed explanation.
     */
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );

    // Click initialization.
    microwave6_cfg_setup( &microwave6_cfg );
    MICROWAVE6_MAP_MIKROBUS( microwave6_cfg, MIKROBUS_1 );
    err_t init_flag = microwave6_init( &microwave6, &microwave6_cfg );
    if ( ( ADC_ERROR == init_flag ) || ( I2C_MASTER_ERROR == init_flag ) )
    {
        log_error( &logger, " Communication init." );
        for ( ; ; );
    }
    
    log_printf( &logger, " Calibrating the sensor...\r\n" );
    log_printf( &logger, " There must be no movement near the sensor!\r\n" );
    log_printf( &logger, "----------------------------------\r\n" );
    Delay_ms( 3000 );
    
    if ( MICROWAVE6_OK == microwave6_read_raw_adc( &microwave6, &reference ) )
    {
        log_printf( &logger, " The sensor has been calibrated!\r\n" );
        log_printf( &logger, " Reference value: %d\r\n", reference );
        log_printf( &logger, "----------------------------------\r\n" );
        Delay_ms( 100 );
    }
    else
    {
        log_error( &logger, " Communication error." );
        for ( ; ; );
    }
    
    log_printf( &logger, "The motion detector unit is ready.\r\n" );
    log_printf( &logger, "----------------------------------\r\n" );
    Delay_ms( 100 );
}

void application_task ( void ) 
{
    if ( ( MICROWAVE6_OK == microwave6_read_raw_adc( &microwave6, &raw_adc ) ) && 
         ( raw_adc > MICROWAVE6_THRESHOLD ) )
    {
        if ( ( ( ( raw_adc + MICROWAVE6_THRESHOLD ) < reference )   || 
               ( ( raw_adc - MICROWAVE6_THRESHOLD ) > reference ) ) && 
                 old_detector != raw_adc )
        {
            log_printf( &logger, " Motion detected!\r\n" );
            log_printf( &logger, " Detector value : %u\r\n", raw_adc );
            log_printf( &logger, "----------------------------------\r\n" );
            old_detector = raw_adc;
            Delay_ms( 100 );
        }
    }
}

void main ( void ) 
{
    application_init( );

    for ( ; ; ) 
    {
        application_task( );
    }
}

// ------------------------------------------------------------------------ END
